package at.reisinger;

import java.sql.*;
import java.util.ArrayList;

/**
 * Stellt die Verbindung zur "Datenbank" her
 * Die "Datenbank" sind hier nur beispieldaten.
 */
public class DBConnection {
    private ArrayList<TradingDeal> tradingDeals;
    private ArrayList<Package> packages;
    private Connection c;
    private Statement stmt;

    private Connection con() {
        this.c = null;
        try {
            Class.forName("org.postgresql.Driver");
            this.c = DriverManager
                    .getConnection("jdbc:postgresql://postgres.dergeorg.at:5432/mtcg",
                            "user", "user");
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println(e.getClass().getName() + ": " + e.getMessage());
            System.exit(0);
        }
        return c;
    }

    /**
     * Erstellt alle Beispieldaten und simuliert somit den
     * Verbindungsaufbau zu einer DB
     */
    public DBConnection() {
        this.c = con();
    }

    /**
     * Erstellt alle Tabellen die für den Betrieb der Software bennötigt werden
     *
     * @throws SQLException
     */
    public void init() {
        this.c = con();
        try {
            stmt = this.c.createStatement();
            String sql = "CREATE TABLE IF NOT EXISTS USERS " +
                    "(username       TEXT PRIMARY KEY     NOT NULL," +
                    " nachname           TEXT    NOT NULL, " +
                    " email            TEXT     NOT NULL, " +
                    " password        TEXT NOT NULL)";
            stmt.executeUpdate(sql);
            stmt.close();
            this.c.close();
        } catch (Exception e) {
            System.err.println(e.getClass().getName() + ": " + e.getMessage());
            System.exit(0);
        }
        System.out.println("User Table created");

        this.c = con();
        try {
            stmt = this.c.createStatement();
            String sql = "CREATE TABLE IF NOT EXISTS CARD(NAME TEXT not null,DAMAGE FLOAT not null,ELEMENTTYP TEXT not null,CARDTYPE TEXT not null, PRIMARY KEY (\"name\"));";
            stmt.executeUpdate(sql);
            stmt.close();
            this.c.close();
        } catch (Exception e) {
            System.err.println(e.getClass().getName() + ": " + e.getMessage());
            System.exit(0);
        }
        this.c = con();
        try {
            stmt = this.c.createStatement();
            String sql = "create unique index IF NOT EXISTS card_name_uindex on CARD (NAME);";
            stmt.executeUpdate(sql);
            stmt.close();
            this.c.close();
        } catch (Exception e) {
            System.err.println(e.getClass().getName() + ": " + e.getMessage());
            System.exit(0);
        }
        System.out.println("Card Table created");

        this.c = con();
        try {
            stmt = this.c.createStatement();
            String sql = "create table IF NOT EXISTS package" +
                    "(" +
                    "    \"ID\" varchar(255) not null," +
                    "    name varchar(255) not null" +
                    "        constraint name" +
                    "            references card," +
                    "    i    serial       not null" +
                    "        constraint package_i" +
                    "            primary key" +
                    ");";
            stmt.executeUpdate(sql);
            stmt.close();
            this.c.close();
        } catch (Exception e) {
            System.err.println(e.getClass().getName() + ": " + e.getMessage());
            System.exit(0);
        }
        System.out.println("Package Table created");
    }

    /**
     * Überprüft anhand des Namen der Karte, ob es schon eine solche Karte gibt
     *
     * @param name Name der zu prüfen ist
     * @return True falls die Karte bereits existiert
     */
    public boolean cardExists(String name) {
        this.c = con();
        int count = 0;
        try {
            stmt = this.c.createStatement();
            ResultSet rs = stmt.executeQuery("select count(*) from card where name = \'" + name + "\';");
            while (rs.next()) {
                count = rs.getInt("count");
                System.out.println("Count: ");
                System.out.print(count);
            }
            rs.close();
            stmt.close();
            this.c.close();
        } catch (Exception e) {
            System.err.println(e.getClass().getName() + ": " + e.getMessage());
            System.exit(0);
        }
        if (count == 1) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Fügt eine Karte der DB hinzu
     * @param card Card die hinzugefügt werden soll
     * @return true bei erfolg, false bei error
     */
    public boolean addCard(Card card) {
        this.c = con();
        try{
            stmt = this.c.createStatement();
            String sql = "insert into card (NAME, DAMAGE, ELEMENTTYP, CARDTYPE) values (\'" + card.getName() +"\',\'" + card.getDamage() + "\',\'" + card.getElementTyp().name() + "\',\'" + card.getCardType().name() + "\')";
            System.out.println(sql);
            stmt.executeUpdate(sql);
            stmt.close();
            this.c.close();
        } catch ( Exception e ) {
            System.err.println( e.getClass().getName()+": "+ e.getMessage() );
            return false;
        }
        System.out.println("Card added");
        return true;
    }

    /**
     * Ermittelt die nächste Package ID
     * @return Die Nächste Package id, wenn 0, dann ist ein error
     */
    public int nextPackageId(){
        this.c = con();
        String id = "";
        try{
            stmt = this.c.createStatement();
            ResultSet rs = stmt.executeQuery( "select max(\"ID\") from package;");
            while ( rs.next() ) {
                id = rs.getString("max");
            }
            if (id == null){
                id = "0";
            }
            rs.close();
            stmt.close();
            this.c.close();
        } catch ( Exception e ) {
            System.err.println( e.getClass().getName()+": "+ e.getMessage() );
            return 0;
        }
        System.out.println("Package Next id is: ");
        System.out.print(id);
        return Integer.parseInt(id) + 1;
    }

    /**
     * Fügt eine Package der DB hinzu,
     * wenn eine Karte aus dem Package noch nicht in der Db ist,
     * dann wird es automatisch hinzugefuegt
     * @param pack Pack welches hinzugefügt werden soll
     * @return True bei erfolg, false bei fehler
     */
    public boolean addPackage(Package pack) {
        for (Card ca : pack.getCards()) {
            if (!cardExists(ca.getName())) {
                if(!addCard(ca)){
                    return false;
                }
            }
            this.c = con();
            try {
                stmt = this.c.createStatement();
                String sql = "INSERT INTO package (\"ID\", \"name\") values (\'" + pack.getName() + "\',\'" + ca.getName() + "\');";
                System.out.println(sql);
                stmt.executeUpdate(sql);
                stmt.close();
                this.c.close();
            } catch (Exception e) {
                System.err.println(e.getClass().getName() + ": " + e.getMessage());
                return false;
            }
            System.out.println("Card zu Package hinzugefügt");
        }
        return true;
    }

    /**
     * Fügt einen User der DB hinzu
     * @param username Username des Users
     * @param password Passwort des Users
     * @param nachname nachname des Users
     * @param email Email des Users
     * @return True wenn alles erfolgreich war, false wenn es nicht erfolgreich war
     */
    public boolean addUser(String username, String password, String nachname, String email){
        this.c = con();
        try{
            stmt = this.c.createStatement();
            String sql = "INSERT INTO users (username, nachname, email, password) values (\'" + username +"\',\'" + nachname + "\',\'" + email + "\',\'" + password + "\')";
            System.out.println(sql);
            stmt.executeUpdate(sql);
            stmt.close();
            this.c.close();
        } catch ( Exception e ) {
            System.err.println( e.getClass().getName()+": "+ e.getMessage() );
            return false;
        }
        System.out.println("User added");
        return true;
    }

    /**
     * Sucht einen User anhand des Usernamens.
     * Gibt ein User Objekt zurück
     * @param uname Username der zu suchen ist
     * @return User als User Objekt
     * @throws SQLException Fehler bei der Suche in der DB
     */
    public User getUser(String uname) throws SQLException {
        this.c = con();
        String username="", password="", email ="", nachname = "";
        try{
        stmt = this.c.createStatement();
        ResultSet rs = stmt.executeQuery( "SELECT * FROM users where username = \'" + uname + "\';" );
        while ( rs.next() ) {
            username = rs.getString("username");
            nachname = rs.getString("nachname");
            email = rs.getString("email");
            password = rs.getString("password");
            System.out.println( "username = " + username );
            System.out.println( "nachname = " + nachname );
            System.out.println( "email = " + email );
            System.out.println( "password = " + password );
            System.out.println();
        }
        rs.close();
        stmt.close();
        this.c.close();
        } catch ( Exception e ) {
            System.err.println( e.getClass().getName()+": "+ e.getMessage() );
            System.exit(0);
        }
          System.out.println("Operation done successfully");
        return new User(new Credentials(username, password), username, username, email, new Coins(20));
    }
    /**
     * Holt alle verfügbaren TradingDeals
     * @return Alle Trading Deals
     */
    public ArrayList<TradingDeal> getAllTradingDeals(){
        return this.tradingDeals;
    }

    /**
     * Holt alle verfügbaren Packages
     * @return Alle Packages
     */
    public ArrayList<Package> getAllPackages(){
        return this.packages;
    }

    /**
     * Fügt einen neuen Tradingdeal zum at.reisinger.Store hinzu
     * @param tradingDeal Der neue Deal
     */
    public void addTradingDeal(TradingDeal tradingDeal){
        this.tradingDeals.add(tradingDeal);
    }

//    /**
//     * Fügt ein neues at.reisinger.Package zu dem at.reisinger.Store hinzu
//     * @param packageDeal Das neue at.reisinger.Package
//     */
//    public void addPackage(at.reisinger.Package packageDeal){
//        this.packages.add(packageDeal);
//    }
}
