package at.reisinger.server.objects;

import at.reisinger.*;
import at.reisinger.Package;
import at.reisinger.server.msg.MsgHandler;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Map;

/**
 * Erstellt und sendet eine Response anhand des Requests
 * @author Georg Reisinger
 */
public class Response {

    //private final Socket socket;
    private final int          id;
    private PrintStream out;
    private String       cmd;
    private String       url;
    private String payload;
    private String authString;
    private MsgHandler msgHandler;
    private StringBuilder rqBuilder;

    /**
     * Nimmt die Daten des requests und generiert eine Response
     * @param id Thread Id
     * @param url Request Url
     * @param cmd Request CMD
     * @param out out Print Stream
     * @param authString Der MsgHandler
     * @param payload Payload des Requests
     */
    public Response(int id, String url, String cmd, PrintStream out, String authString, String payload){
        this.id = id;
        this.authString = authString;
        this.url = url;
        this.cmd = cmd;
        this.out = out;
        this.payload = payload;
        this.rqBuilder = new StringBuilder();
        System.out.println(cmd);
        if (this.url != null) {
            if (this.cmd.equals("GET")) {
                getMethodes();
            }else if (this.cmd.equals("POST")){
                try {
                    postMethodes();
                } catch (JsonProcessingException e) {
                    e.printStackTrace();
                }
            }else if (this.cmd.equals("PUT")){
                putMethodes();
            }else if (this.cmd.equals("DELETE")){
                deleteMethodes();
            }else{
                sendError("405");
            }
        }
    }

    private void getMethodes(){
        if (this.url.startsWith("/messages")) {
            String lastBit = this.url.substring(this.url.lastIndexOf('/') + 1);
            System.out.println("Last Bit: " + lastBit);
            if(lastBit.equals("messages")){
                listAllMsg();
            }else{
                String message = msgHandler.getMsg(Integer.parseInt(lastBit)).getMsg();
                if(message == null){
                    sendError("404");
                }else {
                    sendResponse(message, "200");
                }
            }
        } else if (this.url.startsWith("/")) {
            startseite();
        }
    }

    /**
     * Vergleicht das angegebene PW mit dem gespeicherten PW, True wenn richtig
     * @param username User aus DB
     * @param password PW zu User aus DB
     * @param compareTo Base64 String von Basic Auth
     * @return True if Login is correct, false if incorrect
     */
    private boolean basicAuth(String username, String password, String compareTo){
        String authStringEnc = basicAuthBase64(username, password);
        if (compareTo.equals(authStringEnc)){
            return true;
        }
        return false;
    }

    private String basicAuthBase64(String username, String password){
        String authString = username + ":" + password;
        //System.out.println("auth string: " + authString);
        byte[] authEncBytes = Base64.getEncoder().encode(authString.getBytes());
        String authStringEnc = new String(authEncBytes);
        //System.out.println("Base64 encoded auth string: " + authStringEnc);
        return authStringEnc;
    }

    private String basicAuthGetUsername(String authString) throws UnsupportedEncodingException {
        String[] values;
        if (authString != null) {
            System.out.println("1");
            byte[] credDecoded = Base64.getDecoder().decode(authString);
            System.out.println("2");
            String credentials = new String(credDecoded);
            System.out.println("credentials:   ");
            System.out.println(credentials);
            // credentials = username:password
            values = credentials.split(":", 2);
        }else{
            values = null;
        }
        return values[0];
    }

    private void postMethodes() throws JsonProcessingException {
        if (this.url.startsWith("/users")) {
            //payload to map
            ObjectMapper objectMapper = new ObjectMapper();
            Map<String, Object> map = objectMapper.readValue(this.payload, new TypeReference<Map<String,Object>>(){});
            String username = (String) map.get("Username");
            String password = (String) map.get("Password");
            User newUser = new User(new Credentials(username, password), username, username, "not implemented", new Coins(20));
            System.out.println("CON DB");
            DBConnection con = new DBConnection();
            if(!con.addUser(username, password, username, "not implemented")){
                sendError("409");
            }

            //User to json
            objectMapper = new ObjectMapper();
            String userJson = "";
            if(newUser != null) {
                try {
                    userJson += objectMapper.writeValueAsString(newUser);
                } catch (JsonProcessingException e) {
                    userJson = "Error: JsonProcessingException -> " + e.getMessage();
                }
                sendResponse(userJson, "201");
            }else{
                sendError("500");
            }
        }else if (this.url.startsWith("/sessions")) {
            //payload to map
            ObjectMapper objectMapper = new ObjectMapper();
            Map<String, Object> map = objectMapper.readValue(this.payload, new TypeReference<Map<String,Object>>(){});
            String username = (String) map.get("Username");
            String password = (String) map.get("Password");
            User user = null;

            DBConnection con = new DBConnection();
            try {
                user = con.getUser(username);
            } catch (SQLException throwables) {
                throwables.printStackTrace();
            }

            String authString = basicAuthBase64(user.getCredentials().getUsername(), user.getCredentials().getPasswort());
            if(basicAuth(username, password, authString)){
                sendResponse(authString, "200");
            }else{
                sendError("401");
            }
        }else if (this.url.startsWith("/packages")) {
            if(basicAuth("admin", "admin", this.authString)){
                System.out.println("1");
                ObjectMapper objectMapper = new ObjectMapper();
                System.out.println("2");
                ArrayList<Card> listCards = objectMapper.readValue(this.payload, new TypeReference<ArrayList<Card>>(){});
                System.out.println("List cards");
                System.out.println(listCards.toString());
                Package packageCards = new Package(new Cards(listCards), new DBConnection().nextPackageId()+"", 5);
                System.out.println("3");
                if(!new DBConnection().addPackage(packageCards)){
                    sendError("500");
                }else{
                    //Package to json
                    objectMapper = new ObjectMapper();
                    String packageJson = "";
                    if(packageCards != null) {
                        try {
                            packageJson += objectMapper.writeValueAsString(packageCards);
                        } catch (JsonProcessingException e) {
                            packageJson = "Error: JsonProcessingException -> " + e.getMessage();
                        }
                        sendResponse(packageJson, "201");
                    }else{
                        sendError("500");
                    }
                }
            }else{
                sendError("401");
            }
        }
    }

    private void putMethodes(){
        if (this.url.startsWith("/messages")) {
            String lastBit = this.url.substring(this.url.lastIndexOf('/') + 1);
            System.out.println("Last Bit: " + lastBit);
            System.out.println("Payload" + this.payload);
            String message = msgHandler.editMsg(Integer.parseInt(lastBit), this.payload);
            if(message == null){
                sendError("404");
            }else {
                sendResponse("","200");
            }
        }
    }

    private void deleteMethodes(){
        if (this.url.startsWith("/messages")) {
            String lastBit = this.url.substring(this.url.lastIndexOf('/') + 1);
            String message = msgHandler.delMsg(Integer.parseInt(lastBit));
            if(message == null){
                sendError("404");
            }else {
                sendResponse("", "200");
            }
        }
    }


    /**
     * Sendet einen Error Response
     * @param errorCode Der Error Code
     */
    private void sendError(String errorCode) {
        out.print("HTTP/1.0 "+errorCode+"\r\n");
        out.print("Server: Apache/0.8.4\r\n");
        out.print("Content-Type: text/plain\r\n");
        out.print("Content-Length: 1\r\n");
        out.print("\r\n");
        //out.print(responseText);
    }

    private void startseite() {
        sendResponse("lists all messages:   GET    /messages<br>" +
                "add message:          POST   /messages     (Payload: the message; Response an id like1)<br>" +
                "show first message:   GET    /messages/1<br>" +
                "show third message:   GET    /messages/3<br>" +
                "update first message: PUT    /messages/1   (Payload: the message)<br>" +
                "remove first message: DELETE /messages/1<br>", "200");
    }

    private void listAllMsg() {
        sendResponse(msgHandler.getAllMsg(), "200");
        //sendResponse("Test");
    }

    /**
     * Sendet eine Response
     * @param responseText Text der zu senden ist
     * @param code Http code
     */
    private void sendResponse(String responseText, String code){
        out.print("HTTP/1.0 "+code+"\r\n");
        out.print("Server: Apache/0.8.4\r\n");
        out.print("Content-Type: text/plain\r\n");
        out.print("Content-Length: "+responseText.length()+"\r\n");
        out.print("\r\n");
        out.print(responseText);
    }

    /**
     * Get Msg Handler
     *
     * @return msgHandler Handler der Nachrichten
     */
    public MsgHandler getMsghandler() {
        return this.msgHandler;
    }

}
